from typing import List
from BuildingElement import BuildingElement

import NemAll_Python_BasisElements as AllplanBasisElements
import NemAll_Python_BaseElements as AllplanBaseElements
import NemAll_Python_Geometry as AllplanGeo

def check_allplan_version(build_ele, version):
    """
    Check the current Allplan version

    Args:
        build_ele: the building element.
        version:   the current Allplan version

    Returns:
        True/False if version is supported by this script
    """

    # Delete unused arguments
    del build_ele
    del version

    # Support all versions
    return True

def create_element(build_ele, doc):
    """
    Creation of element

    Args:
        build_ele: the building element.
        doc:       input document
    """
    single_foundation = SingleFoundation()
    return (single_foundation.create_models(build_ele), [])

class SingleFoundation():
    """
    Definition of class single foundation
    """

    def __init__(self):
        """
        Initialisation of class single foundation
        """
        pass

    def create_models(self, build_ele: BuildingElement) -> List[AllplanBasisElements.ModelElement3D]:
        """
        Create all the models contain foundation and column

        Args:
            build_ele: the building element

        Return: List of element
        """
        elements = []

        elements += self.create_foundation(build_ele)
        elements += self.create_column(build_ele)

        return elements

    def create_foundation(self, build_ele: BuildingElement) -> List[AllplanBasisElements.ModelElement3D]:
        """
        Create foundation model

        Args:
            build_ele: the building element

        Return: List of ModelElement3D
        """
        common_prop = self.get_foundation_common_props(build_ele)
        geometry = AllplanGeo.Polyhedron3D.CreateCuboid(build_ele.FounLength.value, #length
                                                        build_ele.FounWidth.value,  #width
                                                        build_ele.FounHeight.value) #height
        return [AllplanBasisElements.ModelElement3D(common_prop, geometry)]

    def create_column(self, build_ele: BuildingElement) -> List[AllplanBasisElements.ModelElement3D]:
        """
        Create column model

        Args:
            build_ele: the building element

        Return: List of ModelElement3D
        """
        common_prop = self.get_column_common_props(build_ele)
        geometry = AllplanGeo.Polyhedron3D.CreateCuboid(build_ele.ColumnLength.value, #length
                                                        build_ele.ColumnWidth.value,  #width
                                                        build_ele.ColumnHeight.value) #height

        geometry = self.move_column_to_center(build_ele, geometry)
        return [AllplanBasisElements.ModelElement3D(common_prop, geometry)]

    def get_foundation_common_props(self, build_ele: BuildingElement) -> AllplanBaseElements.CommonProperties:
        """
        Get foundation common properties

        Return: AllplanBaseElements.CommonProperties
        """

        common_prop = AllplanBaseElements.CommonProperties()

        common_prop.Color   = build_ele.FounColor.value
        common_prop.Pen     = build_ele.FounPen.value
        common_prop.Stroke  = build_ele.FounStroke.value
        common_prop.Layer   = build_ele.FounLayer.value

        #use global properties
        if build_ele.IsFounUseGlobalProp.value:
            common_prop.GetGlobalProperties()

        return common_prop

    def get_column_common_props(self, build_ele: BuildingElement) -> AllplanBaseElements.CommonProperties:
        """
        Get column common properties

        Return: AllplanBaseElements.CommonProperties
        """

        common_prop = AllplanBaseElements.CommonProperties()

        common_prop.Color   = build_ele.ColumnColor.value
        common_prop.Pen     = build_ele.ColumnPen.value
        common_prop.Stroke  = build_ele.ColumnStroke.value
        common_prop.Layer   = build_ele.ColumnLayer.value

        #use global properties
        if build_ele.IsFounUseGlobalProp.value:
            common_prop.GetGlobalProperties()

        return common_prop

    def move_column_to_center(self, build_ele: BuildingElement, column: AllplanGeo.Polyhedron3D) -> AllplanGeo.Polyhedron3D:
        """move column to center of foundation"""
        vector = AllplanGeo.Vector3D(build_ele.FounLength.value/2 - build_ele.ColumnLength.value/2,
                                     build_ele.FounWidth.value/2 - build_ele.ColumnWidth.value/2,
                                     build_ele.FounHeight.value)

        return AllplanGeo.Move(column, vector)